@@ -201,139 +201,6 @@ atspi_accessible_class_init (AtspiAccessibleClass *klass)
   quark_locale = g_quark_from_string ("accessible-locale");
 }
 
-/* TODO: Generate following from spec? */
-static const char *role_names [] =
-{
-  "invalid",
-  "accelerator label",
-  "alert",
-  "animation",
-  "arrow",
-  "calendar",
-  "canvas",
-  "check box",
-  "check menu item",
-  "color chooser",
-  "column header",
-  "combo box",
-  "dateeditor",
-  "desktop icon",
-  "desktop frame",
-  "dial",
-  "dialog",
-  "directory pane",
-  "drawing area",
-  "file chooser",
-  "filler",
-  "focus traversable",
-  "fontchooser",
-  "frame",
-  "glass pane",
-  "html container",
-  "icon",
-  "image",
-  "internal frame",
-  "label",
-  "layered pane",
-  "list",
-  "list item",
-  "menu",
-  "menu bar",
-  "menu item",
-  "option pane",
-  "page tab",
-  "page tab list",
-  "panel",
-  "password text",
-  "popup menu",
-  "progress bar",
-  "push button",
-  "radio button",
-  "radio menu item",
-  "root pane",
-  "row header",
-  "scroll bar",
-  "scroll pane",
-  "separator",
-  "slider",
-  "spin button",
-  "split pane",
-  "statusbar",
-  "table",
-  "table cell",
-  "table column header",
-  "table row header",
-  "tear off menu item",
-  "terminal",
-  "text",
-  "toggle button",
-  "tool bar",
-  "tool tip",
-  "tree",
-  "tree table",
-  "unknown",
-  "viewport",
-  "window",
-  NULL,
-  "header",
-  "footer",
-  "paragraph",
-  "ruler",
-  "application",
-  "autocomplete",
-  "editbar",
-  "embedded component",
-  "entry",
-  "chart",
-  "caption",
-  "document frame",
-  "heading",
-  "page",
-  "section",
-  "redundant object",
-  "form",
-  "link",
-  "input method window",
-  "table row",
-  "tree item",
-  "document spreadsheet",
-  "document presentation",
-  "document text",
-  "document web",
-  "document email",
-  "comment",
-  "list box",
-  "grouping",
-  "image map",
-  "notification",
-  "info bar",
-  "level bar"
-};
-
-#define MAX_ROLES (sizeof (role_names) / sizeof (char *))
-
-/**
- * atspi_role_get_name:
- * @role: an #AtspiRole object to query.
- *
- * Gets a localizable string that indicates the name of an #AtspiRole.
- * <em>DEPRECATED.</em>
- *
- * Returns: a localizable string name for an #AtspiRole enumerated type.
- **/
-gchar *
-atspi_role_get_name (AtspiRole role)
-{
-  if (role < MAX_ROLES && role_names [(int) role])
-    {
-      return g_strdup (role_names [(int) role]);
-    }
-  else
-    {
-      return g_strdup ("");
-    }
-}
-
 /**
  * atspi_accessible_get_name:
  * @obj: a pointer to the #AtspiAccessible object on which to operate.
@@ -621,14 +488,14 @@ atspi_accessible_get_role (AtspiAccessible *obj, GError **error)
 gchar *
 atspi_accessible_get_role_name (AtspiAccessible *obj, GError **error)
 {
-  char *retval = NULL;
+  gchar *retval = NULL;
   AtspiRole role;
 
   g_return_val_if_fail (obj != NULL, NULL);
 
   role = atspi_accessible_get_role (obj, error);
-  if (role >= 0 && role < MAX_ROLES && role != ATSPI_ROLE_EXTENDED)
-    return g_strdup (role_names [role]);
+  if (role >= 0 && role < ATSPI_ROLE_COUNT && role != ATSPI_ROLE_EXTENDED)
+    return atspi_role_get_name (role);
 
   _atspi_dbus_call (obj, atspi_interface_accessible, "GetRoleName", error, "=>s", &retval);
 
@@ -1205,6 +1072,9 @@ atspi_accessible_is_value (AtspiAccessible *obj)
  *
  * Returns: (transfer full): a pointer to an #AtspiAction interface
  *          instance, or NULL if @obj does not implement #AtspiAction.
+ *
+ * Deprecated: 2.10: Use atspi_accessible_get_action_iface instead.
+ * Rename to: atspi_accessible_get_action_iface
  **/
 AtspiAction *
 atspi_accessible_get_action (AtspiAccessible *accessible)
@@ -1213,6 +1083,22 @@ atspi_accessible_get_action (AtspiAccessible *accessible)
           g_object_ref (ATSPI_ACTION (accessible)) : NULL);  
 }
 
+/**
+ * atspi_accessible_get_action_iface:
+ * @obj: a pointer to the #AtspiAccessible instance to query.
+ *
+ * Gets the #AtspiAction interface for an #AtspiAccessible.
+ *
+ * Returns: (transfer full): a pointer to an #AtspiAction interface
+ *          instance, or NULL if @obj does not implement #AtspiAction.
+ **/
+AtspiAction *
+atspi_accessible_get_action_iface (AtspiAccessible *accessible)
+{
+  return (_atspi_accessible_is_a (accessible, atspi_interface_action) ?
+          g_object_ref (ATSPI_ACTION (accessible)) : NULL);  
+}
+
 /**
  * atspi_accessible_get_collection:
  * @obj: a pointer to the #AtspiAccessible instance to query.
@@ -1221,6 +1107,9 @@ atspi_accessible_get_action (AtspiAccessible *accessible)
  *
  * Returns: (transfer full): a pointer to an #AtspiCollection interface
  *          instance, or NULL if @obj does not implement #AtspiCollection.
+ *
+ * Deprecated: 2.10: Use atspi_accessible_get_collection_iface instead.
+ * Rename to: atspi_accessible_get_collection_iface
  **/
 AtspiCollection *
 atspi_accessible_get_collection (AtspiAccessible *accessible)
@@ -1229,6 +1118,22 @@ atspi_accessible_get_collection (AtspiAccessible *accessible)
           g_object_ref (ATSPI_COLLECTION (accessible)) : NULL);  
 }
 
+/**
+ * atspi_accessible_get_collection_iface:
+ * @obj: a pointer to the #AtspiAccessible instance to query.
+ *
+ * Gets the #AtspiCollection interface for an #AtspiAccessible.
+ *
+ * Returns: (transfer full): a pointer to an #AtspiCollection interface
+ *          instance, or NULL if @obj does not implement #AtspiCollection.
+ **/
+AtspiCollection *
+atspi_accessible_get_collection_iface (AtspiAccessible *accessible)
+{
+  return (_atspi_accessible_is_a (accessible, atspi_interface_collection) ?
+          g_object_ref (ATSPI_COLLECTION (accessible)) : NULL);  
+}
+
 /**
  * atspi_accessible_get_component:
  * @obj: a pointer to the #AtspiAccessible instance to query.
@@ -1237,6 +1142,9 @@ atspi_accessible_get_collection (AtspiAccessible *accessible)
  *
  * Returns: (transfer full): a pointer to an #AtspiComponent interface
  *          instance, or NULL if @obj does not implement #AtspiComponent.
+ *
+ * Deprecated: 2.10: Use atspi_accessible_get_component_iface instead.
+ * Rename to: atspi_accessible_get_component_iface
  **/
 AtspiComponent *
 atspi_accessible_get_component (AtspiAccessible *obj)
@@ -1245,6 +1153,22 @@ atspi_accessible_get_component (AtspiAccessible *obj)
           g_object_ref (ATSPI_COMPONENT (obj)) : NULL);
 }
 
+/**
+ * atspi_accessible_get_component_iface:
+ * @obj: a pointer to the #AtspiAccessible instance to query.
+ *
+ * Gets the #AtspiComponent interface for an #AtspiAccessible.
+ *
+ * Returns: (transfer full): a pointer to an #AtspiComponent interface
+ *          instance, or NULL if @obj does not implement #AtspiComponent.
+ **/
+AtspiComponent *
+atspi_accessible_get_component_iface (AtspiAccessible *obj)
+{
+  return (_atspi_accessible_is_a (obj, atspi_interface_component) ?
+          g_object_ref (ATSPI_COMPONENT (obj)) : NULL);
+}
+
 /**
  * atspi_accessible_get_document:
  * @obj: a pointer to the #AtspiAccessible instance to query.
@@ -1253,6 +1177,9 @@ atspi_accessible_get_component (AtspiAccessible *obj)
  *
  * Returns: (transfer full): a pointer to an #AtspiDocument interface
  *          instance, or NULL if @obj does not implement #AtspiDocument.
+ *
+ * Deprecated: 2.10: Use atspi_accessible_get_document_iface instead.
+ * Rename to: atspi_accessible_get_document_iface
  **/
 AtspiDocument *
 atspi_accessible_get_document (AtspiAccessible *accessible)
@@ -1261,6 +1188,22 @@ atspi_accessible_get_document (AtspiAccessible *accessible)
           g_object_ref (ATSPI_DOCUMENT (accessible)) : NULL);  
 }
 
+/**
+ * atspi_accessible_get_document_iface:
+ * @obj: a pointer to the #AtspiAccessible instance to query.
+ *
+ * Gets the #AtspiDocument interface for an #AtspiAccessible.
+ *
+ * Returns: (transfer full): a pointer to an #AtspiDocument interface
+ *          instance, or NULL if @obj does not implement #AtspiDocument.
+ **/
+AtspiDocument *
+atspi_accessible_get_document_iface (AtspiAccessible *accessible)
+{
+  return (_atspi_accessible_is_a (accessible, atspi_interface_document) ?
+          g_object_ref (ATSPI_DOCUMENT (accessible)) : NULL);  
+}
+
 /**
  * atspi_accessible_get_editable_text:
  * @obj: a pointer to the #AtspiAccessible instance to query.
@@ -1269,6 +1212,9 @@ atspi_accessible_get_document (AtspiAccessible *accessible)
  *
  * Returns: (transfer full): a pointer to an #AtspiEditableText interface
  *          instance, or NULL if @obj does not implement #AtspiEditableText.
+ *
+ * Deprecated: 2.10: Use atspi_accessible_get_editable_text_iface instead.
+ * Rename to: atspi_accessible_get_editable_text_iface
  **/
 AtspiEditableText *
 atspi_accessible_get_editable_text (AtspiAccessible *accessible)
@@ -1277,6 +1223,22 @@ atspi_accessible_get_editable_text (AtspiAccessible *accessible)
           g_object_ref (ATSPI_EDITABLE_TEXT (accessible)) : NULL);  
 }
 
+/**
+ * atspi_accessible_get_editable_text_iface:
+ * @obj: a pointer to the #AtspiAccessible instance to query.
+ *
+ * Gets the #AtspiEditableText interface for an #AtspiAccessible.
+ *
+ * Returns: (transfer full): a pointer to an #AtspiEditableText interface
+ *          instance, or NULL if @obj does not implement #AtspiEditableText.
+ **/
+AtspiEditableText *
+atspi_accessible_get_editable_text_iface (AtspiAccessible *accessible)
+{
+  return (_atspi_accessible_is_a (accessible, atspi_interface_editable_text) ?
+          g_object_ref (ATSPI_EDITABLE_TEXT (accessible)) : NULL);  
+}
+
 /**
  * atspi_accessible_get_hyperlink:
  * @obj: a pointer to the #AtspiAccessible object on which to operate.
@@ -1301,6 +1263,9 @@ atspi_accessible_get_hyperlink (AtspiAccessible *accessible)
  *
  * Returns: (transfer full): a pointer to an #AtspiHypertext interface
  *          instance, or NULL if @obj does not implement #AtspiHypertext.
+ *
+ * Deprecated: 2.10: Use atspi_accessible_get_hypertext_iface instead.
+ * Rename to: atspi_accessible_get_hypertext_iface
  **/
 AtspiHypertext *
 atspi_accessible_get_hypertext (AtspiAccessible *accessible)
@@ -1309,6 +1274,22 @@ atspi_accessible_get_hypertext (AtspiAccessible *accessible)
           g_object_ref (ATSPI_HYPERTEXT (accessible)) : NULL);  
 }
 
+/**
+ * atspi_accessible_get_hypertext_iface:
+ * @obj: a pointer to the #AtspiAccessible instance to query.
+ *
+ * Gets the #AtspiHypertext interface for an #AtspiAccessible.
+ *
+ * Returns: (transfer full): a pointer to an #AtspiHypertext interface
+ *          instance, or NULL if @obj does not implement #AtspiHypertext.
+ **/
+AtspiHypertext *
+atspi_accessible_get_hypertext_iface (AtspiAccessible *accessible)
+{
+  return (_atspi_accessible_is_a (accessible, atspi_interface_hypertext) ?
+          g_object_ref (ATSPI_HYPERTEXT (accessible)) : NULL);  
+}
+
 /**
  * atspi_accessible_get_image:
  * @obj: a pointer to the #AtspiAccessible instance to query.
@@ -1317,6 +1298,9 @@ atspi_accessible_get_hypertext (AtspiAccessible *accessible)
  *
  * Returns: (transfer full): a pointer to an #AtspiImage interface instance, or
  *          NULL if @obj does not implement #AtspiImage.
+ *
+ * Deprecated: 2.10: Use atspi_accessible_get_image_iface instead.
+ * Rename to: atspi_accessible_get_image_iface
  **/
 AtspiImage *
 atspi_accessible_get_image (AtspiAccessible *accessible)
@@ -1325,6 +1309,22 @@ atspi_accessible_get_image (AtspiAccessible *accessible)
           g_object_ref (ATSPI_IMAGE (accessible)) : NULL);  
 }
 
+/**
+ * atspi_accessible_get_image_iface:
+ * @obj: a pointer to the #AtspiAccessible instance to query.
+ *
+ * Gets the #AtspiImage interface for an #AtspiAccessible.
+ *
+ * Returns: (transfer full): a pointer to an #AtspiImage interface instance, or
+ *          NULL if @obj does not implement #AtspiImage.
+ **/
+AtspiImage *
+atspi_accessible_get_image_iface (AtspiAccessible *accessible)
+{
+  return (_atspi_accessible_is_a (accessible, atspi_interface_image) ?
+          g_object_ref (ATSPI_IMAGE (accessible)) : NULL);  
+}
+
 /**
  * atspi_accessible_get_selection:
  * @obj: a pointer to the #AtspiAccessible instance to query.
@@ -1333,6 +1333,9 @@ atspi_accessible_get_image (AtspiAccessible *accessible)
  *
  * Returns: (transfer full): a pointer to an #AtspiSelection interface
  *          instance, or NULL if @obj does not implement #AtspiSelection.
+ *
+ * Deprecated: 2.10: Use atspi_accessible_get_selection_iface instead.
+ * Rename to: atspi_accessible_get_selection_iface
  **/
 AtspiSelection *
 atspi_accessible_get_selection (AtspiAccessible *accessible)
@@ -1341,6 +1344,22 @@ atspi_accessible_get_selection (AtspiAccessible *accessible)
           g_object_ref (ATSPI_SELECTION (accessible)) : NULL);  
 }
 
+/**
+ * atspi_accessible_get_selection_iface:
+ * @obj: a pointer to the #AtspiAccessible instance to query.
+ *
+ * Gets the #AtspiSelection interface for an #AtspiAccessible.
+ *
+ * Returns: (transfer full): a pointer to an #AtspiSelection interface
+ *          instance, or NULL if @obj does not implement #AtspiSelection.
+ **/
+AtspiSelection *
+atspi_accessible_get_selection_iface (AtspiAccessible *accessible)
+{
+  return (_atspi_accessible_is_a (accessible, atspi_interface_selection) ?
+          g_object_ref (ATSPI_SELECTION (accessible)) : NULL);  
+}
+
 #if 0
 /**
  * atspi_accessible_get_streamable_content:
@@ -1367,6 +1386,9 @@ atspi_accessible_get_streamable_content (AtspiAccessible *accessible)
  *
  * Returns: (transfer full): a pointer to an #AtspiTable interface instance, or
  *          NULL if @obj does not implement #AtspiTable.
+ *
+ * Deprecated: 2.10: Use atspi_accessible_get_table_iface instead.
+ * Rename to: atspi_accessible_get_table_iface
  **/
 AtspiTable *
 atspi_accessible_get_table (AtspiAccessible *obj)
@@ -1375,6 +1397,22 @@ atspi_accessible_get_table (AtspiAccessible *obj)
           g_object_ref (ATSPI_TABLE (obj)) : NULL);  
 }
 
+/**
+ * atspi_accessible_get_table_iface:
+ * @obj: a pointer to the #AtspiAccessible instance to query.
+ *
+ * Gets the #AtspiTable interface for an #AtspiAccessible.
+ *
+ * Returns: (transfer full): a pointer to an #AtspiTable interface instance, or
+ *          NULL if @obj does not implement #AtspiTable.
+ **/
+AtspiTable *
+atspi_accessible_get_table_iface (AtspiAccessible *obj)
+{
+  return (_atspi_accessible_is_a (obj, atspi_interface_table) ?
+          g_object_ref (ATSPI_TABLE (obj)) : NULL);  
+}
+
 /**
  * atspi_accessible_get_text:
  * @obj: a pointer to the #AtspiAccessible instance to query.
@@ -1383,6 +1421,9 @@ atspi_accessible_get_table (AtspiAccessible *obj)
  *
  * Returns: (transfer full): a pointer to an #AtspiText interface instance, or
  *          NULL if @obj does not implement #AtspiText.
+ *
+ * Deprecated: 2.10: Use atspi_accessible_get_text_iface instead.
+ * Rename to: atspi_accessible_get_text_iface
  **/
 AtspiText *
 atspi_accessible_get_text (AtspiAccessible *obj)
@@ -1391,6 +1432,22 @@ atspi_accessible_get_text (AtspiAccessible *obj)
           g_object_ref (ATSPI_TEXT (obj)) : NULL);
 }
 
+/**
+ * atspi_accessible_get_text_iface:
+ * @obj: a pointer to the #AtspiAccessible instance to query.
+ *
+ * Gets the #AtspiTable interface for an #AtspiAccessible.
+ *
+ * Returns: (transfer full): a pointer to an #AtspiText interface instance, or
+ *          NULL if @obj does not implement #AtspiText.
+ **/
+AtspiText *
+atspi_accessible_get_text_iface (AtspiAccessible *obj)
+{
+  return (_atspi_accessible_is_a (obj, atspi_interface_text) ?
+          g_object_ref (ATSPI_TEXT (obj)) : NULL);
+}
+
 /**
  * atspi_accessible_get_value:
  * @obj: a pointer to the #AtspiAccessible instance to query.
@@ -1399,6 +1456,9 @@ atspi_accessible_get_text (AtspiAccessible *obj)
  *
  * Returns: (transfer full): a pointer to an #AtspiValue interface instance, or
  *          NULL if @obj does not implement #AtspiValue.
+ *
+ * Deprecated: 2.10: Use atspi_accessible_get_value_iface instead.
+ * Rename to: atspi_accessible_get_value_iface
  **/
 AtspiValue *
 atspi_accessible_get_value (AtspiAccessible *accessible)
@@ -1407,6 +1467,22 @@ atspi_accessible_get_value (AtspiAccessible *accessible)
           g_object_ref (ATSPI_VALUE (accessible)) : NULL);  
 }
 
+/**
+ * atspi_accessible_get_value_iface:
+ * @obj: a pointer to the #AtspiAccessible instance to query.
+ *
+ * Gets the #AtspiTable interface for an #AtspiAccessible.
+ *
+ * Returns: (transfer full): a pointer to an #AtspiValue interface instance, or
+ *          NULL if @obj does not implement #AtspiValue.
+ **/
+AtspiValue *
+atspi_accessible_get_value_iface (AtspiAccessible *accessible)
+{
+  return (_atspi_accessible_is_a (accessible, atspi_interface_value) ?
+          g_object_ref (ATSPI_VALUE (accessible)) : NULL);  
+}
+
 static void
 append_const_val (GArray *array, const gchar *val)
 {
@@ -1480,7 +1556,7 @@ _atspi_accessible_new (AtspiApplication *app, const gchar *path)
  * atspi_accessible_set_cache_mask:
  * @accessible: The #AtspiAccessible to operate on.  Must be the desktop or
  *             the root of an application.
- * @mask: (type int): An #AtspiCache specifying a bit mask of the types of data to cache.
+ * @mask: An #AtspiCache specifying a bit mask of the types of data to cache.
  *
  * Sets the type of data to cache for accessibles.
  * If this is not set for an application or is reset to ATSPI_CACHE_UNDEFINED,
